C           CROWHST
C*********************************************
c This coding has been developed by A.G. EVERY
C*********************************************
C     PROGRAM FOR CALCULATING k-SPACE SURFACE GREEN'S FUNCTIONS
C     OF TWO JOINED ELASTIC HALF SPACES FOR BRILLOUIN SCATTERING.
c     Slight changes to subroutine hval required for non symmetry plane
c Also being used to calculate the SBS spectrum for a water loaded
c surface, showing the leaky RW and Scholte wave
C********************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 ATR(3,3),CU(3,3,3,3),DEL(3,3),EPS(3,3,3)
      REAL*8 CCCU(3,3,3,3),BU(0:6,0:6,0:6),CMU(6,6)
      REAL*8 CL(3,3,3,3)
      REAL*8 CCCL(3,3,3,3),BL(0:6,0:6,0:6),CML(6,6)
	  real*8 intns
      INTEGER ipar(2)
      common /cipar/ ipar
      common /cdep/ del,eps
c     ===============================
      do 543 i=1,6
      do 543 j=1,6
      cmu(i,j)=0.0d0      
      cml(i,j)=0.0d0
543   continue
C     ========4=======================
      rt12=dsqrt(1.0d0/2.0d0)
      rt13=dsqrt(1.0d0/3.0d0)
      rt16=dsqrt(1.0d0/6.0d0)
      PI=3.141592653589793D0
      twpi=2.0d0*pi      
c ***************************
c     100 crystal, x1 axis at angle pph (in degrees) to 100 direction
c     x2 axis at angle pph to 010 direction      
c     x3 axis in 001 direction
      pph=0.1d0*pi/180.0d0
      cpph=dcos(pph)
      spph=dsin(pph)
      atr(1,1)=cpph
      atr(1,2)=spph
      atr(1,3)=0.0d0
      atr(2,1)=-spph
      atr(2,2)=cpph
      atr(2,3)=0.0d0
      atr(3,1)=0.0d0
      atr(3,2)=0.0d0
      atr(3,3)=1.0d0        
c ***************************
c     Materials constants. Avoid exact isotropy.
c ***************************
C     SILICON ELASTIC CONSTANTS AND DENSITY
c      RHOU=2.332D0
c      CMU(1,1)=165.0D0
c      CMU(1,2)=63.0D0
c      CMU(4,4)=79.1D0
C  =======================
C     NICKEL ELASTIC CONSTANTS AND DENSITY c11=247
c      RHOU=9.040D0
c      CMU(1,1)=247.0D0        
c      CMU(1,1)=400.0D0  
c      CMU(1,2)=153.0D0            
c      CMU(4,4)=122.0D0
C  =======================
C     ALUMINIUM ELASTIC CONSTANTS AND DENSITY
c      RHOU=2.70D0
c      CMU(1,1)=110.0D0  
c      CMU(1,2)=55.5D0
c      CMU(4,4)=27.0D0
c =======================
c Stonley values
c      RHOu=5.0D0
c      CMu(1,1)=600.0d0
c      CMu(1,2)=199.0D0
c      CMu(4,4)=200.0D0
C  =======================
c Stonley-Scholte values. Water
c      RHOu=1.0D0
c      CMu(1,1)=2.184d0
c      CMu(4,4)=0.00010D0
c      CMu(1,2)=(cmu(1,1)-2.0d0*cmu(4,4))*0.999d0
c Mozhaev Water
      RHOu=1.0D0
      CMu(1,1)=2.12d0
      CMu(4,4)=0.0000010D0
      CMu(1,2)=(cmu(1,1)-2.0d0*cmu(4,4))*0.999d0
C  =======================
      cmu(1,3)=cmu(1,2)
      cmu(2,2)=cmu(1,1)
      cmu(2,3)=cmu(1,2)
      cmu(3,3)=cmu(1,1)                  
      cmu(5,5)=cmu(4,4)
      cmu(6,6)=cmu(4,4)
      cmu(3,1)=cmu(1,2)
      cmu(2,1)=cmu(1,2)
      cmu(3,2)=cmu(1,2)
c     ==============================
C     LOWER HALFSPACE ELASTIC CONSTANTS AND DENSITY
c     Hypothetical silicon-like
c      RHOL=2.332D0/1.0d0
c      CML(1,1)=165.0D0/1.0d0
c      CML(1,2)=63.0D0/1.0d0
c      CML(4,4)=79.1D0/1.0d0
C  =======================
C     Mozhaev Alloy
      w=0.75d0
      rhoag=10.49d0
      c11ag=139.5d0
      c44ag=27.0d0
      rhoau=19.3d0
      c11au=207.0d0
      c44au=28.5d0
      RHOL=w*rhoag+(1.0d0-w)*rhoau
      CML(1,1)=w*c11ag+(1.0d0-w)*c11au
      CML(4,4)=w*c44ag+(1.0d0-w)*c44au
      CML(1,2)=(cml(1,1)-2.0d0*cml(4,4))*0.999d0
      write(6,*)dsqrt(cml(4,4)/rhol)
C  =======================
C     LOWER HALFSPACE ELASTIC CONSTANTS AND DENSITY
c     Silica constants from FLS.       
c      RHOL=2.600D0
c      CML(1,1)=63.0d0
c      CML(1,2)=20.0D0
c      CML(4,4)=20.0d0
C  =======================
C     LOWER HALFSPACE ELASTIC CONSTANTS AND DENSITY
c     BaF  constants        
c      RHOL=4.8900D0
c      CML(1,1)=90.7d0
c      CML(1,2)=41.0D0
c      CML(4,4)=25.3d0
C  =======================
c Stonley values 
c      RHOL=15.0D0
c      CML(1,1)=1800.0D0
c      CML(1,2)=597.0D0
c      CML(4,4)=600.0D0
c      write(6,*)cml(4,4)/cmu(4,4),rhol/rhou
C  =======================
c Stonley-Scholte B2900 values 
c      RHOL=2.0D0
c      CML(1,1)=16.936D0
c      CML(4,4)=5.249D0
c      CML(1,2)=(cml(1,1)-2.0d0*cml(4,4))*0.999d0
C  =======================
c Stonley-Scholte PVC values 
c      RHOL=1.36D0
c      CML(1,1)=6.996D0
c      CML(4,4)=1.646D0
c      CML(1,2)=(cml(1,1)-2.0d0*cml(4,4))*0.999d0
C  =======================
c Stonley-Scholte hypothetical material values 
c      RHOL=2.0D0
c      CML(1,1)=16.936D0*4.0d0
c      CML(4,4)=5.249D0*4.0d0
c      CML(1,2)=(cml(1,1)-2.0d0*cml(4,4))*0.999d0
C  =======================
      cml(1,3)=cml(1,2)      
      cml(2,2)=cml(1,1)     
      cml(2,3)=cml(1,2)           
      cml(3,3)=cml(1,1)
      cml(5,5)=cml(4,4)
      cml(6,6)=cml(4,4)
      cml(3,1)=cml(1,2)
      cml(2,1)=cml(1,2)
      cml(3,2)=cml(1,2)
c     ==============================
      do 987 i=1,6
      do 987 j=1,6
      cmu(i,j)=cmu(i,j)/rhou 
      cml(i,j)=cml(i,j)/rhol
 987  continue
C     ===================================
C     ELASTIC CONSTANTS ORIGINALLY IN GPA AND RHO IN G/CM**3
C     C(I,J)ARE NOW ELASTIC CONSTANTS DIVIDED BY DENSITY
C     VELOCITIES COME OUT IN MM/MICROSEC
C    =======================================================
C     i and j components of Green's function gij
      IPAR(1)=3
      IPAR(2)=3
c     ===============================
c     Initialising various arrays
      CALL SINIT(atr,bu,cmu,cccu,cu)      
      CALL SINIT(atr,bl,cml,cccl,cl)
c     ==============================
C     MAIN CALCULATION. 
      CALL MODECONV(bu,bl,cccu,cccl,cu,cl,rhou,rhol)
c     ==================================
      STOP
      END


      SUBROUTINE MODECONV(bu,bl,cccu,cccl,cu,cl,rhou,rhol)
C     ***************************************************************
C     A SEQUENCE OF VALUES OF S// ARE GENERATED AND THE EQUATION
C     OF THE SLOWNESS SURFACE IS THEN SOLVED TO DETERMINE THE
C     CORRESPONDING VALUES OF SZ.
c     ***************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 CU(3,3,3,3),CL(3,3,3,3),DEL(3,3),EPS(3,3,3),BS(240)
      REAL*8 CCCU(3,3,3,3),CCCL(3,3,3,3)
      REAL*8 BU(0:6,0:6,0:6),k1,k2
      complex*16 S1P(0:6),S2P(0:6),S3PU(0:6),S3PL(0:6)   
      REAL*8 BL(0:6,0:6,0:6)
	  real*8 intns
      COMPLEX*16 XII,s1,s2,hnw
      INTEGER ipar(2)
      common /cipar/ ipar
      common /cdep/ del,eps
C  ==============================
      xii=(0.0d0,1.0d0)
      S1P(0)=(1.0D0,0.0d0)
      S2P(0)=(1.0D0,0.0d0)
      S3PU(0)=(1.0D0,0.0d0)       
      S3PL(0)=(1.0D0,0.0d0) 
C
      PI=3.141592653589793D0
      twpi=2.0d0*pi      
      do 111 i=1,240
      bs(i)=0.0d0
111   continue
      open(unit=10,file="mozh.dat",status="old") 
c      rrr=rrand()
      damping=0.000001d0
c For true Stoneley waves you would need finite damping    
c      damping=0.0d0
      damping=damping*rhou
      tht=69.4d0*pi/180.0d0
c     mean frequency in GHz and +- fractional variation
c      fmean=30.0d0
c      fract=0.6d0
c      fmin=fmean*(1.0d0-fract)*twpi*1.0d9
c      fmax=fmean*(1.0d0+fract)*twpi*1.0d9
      fmax=5.6d0*twpi*1.0d9
      fmin=4.7d0*twpi*1.0d9
      fdel=fmax-fmin
c
c     iap and jap refer to points in the aperture 
      do 750 iap=1,1
      do 750 jap=1,1
c      rrr=rnd()
c     coding to generate k1 and k2 in mm*-1
      k1=2.0d0*dsin(tht)*(twpi/514.5d-6)*(1.0d0+0.00001d0)
      k2=2.0d0*dsin(tht)*(twpi/514.5d-6)*0.00001d0
      do 75 i=1,240
c     Generating spectrum for particular k1 and k2
c     s1 and s2 im microsec per mm
      fff=fmin+fdel*(i-0.05d0)/dfloat(240)  
      s1=1.0d6*k1/fff
      s2=1.0d6*k2/fff  
c     ==================
c  Calculation of the Green's function hnw
      call hval(hnw,bu,bl,cccu,cccl,cu,cl,rhou,rhol,s1,s2,
     1 s1p,s2p,s3pu,s3pl)
	 intns=0.0
	  mflg=1
	     call hvalhs(s1,s2,fff,intns,
     1    bu,bl,cu,cl,rhou,rhol,damping,mflg)
c     ==================
c     Brillouin spectrun bs(i)=damped Gij/frequency squared
      bs(i)=bs(i)+100.0d0*dimag(xii/(fff*fff*(1.0d0/hnw+damping)))
75    CONTINUE
750   CONTINUE
      Tegrat=0.0d0
      do 222 i=1,240
      fff=fmin+fdel*(i-0.05d0)/dfloat(240)
c for scattering intensity vs. frequency(GHz)
c      write(10,999)fff*1.0d-9/twpi,bs(i)*1.0d22
c for scattering intensity vs. velocity (m/s)
      fff=fff*1.0d-9/twpi
      fffv=fff*514.5d0*0.5d0/dsin(tht)
      write(10,999)fffv,bs(i)*1.0d22,intns
      tegrat=tegrat+bs(i)*1.0d22*2.48d0/240.0d0   
222   continue
      write(6,*)tegrat
999   FORMAT(1X,f10.3,F12.2,f12.2)
      RETURN
      END

      subroutine hval(hnw,bu,bl,cccu,cccl,cu,cl,rhou,rhol,s1,s2,
     1 s1p,s2p,s3pu,s3pl)      
c*************************      
c     Calculation of hnw
c*************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 CU(3,3,3,3),CL(3,3,3,3),DEL(3,3),EPS(3,3,3)
      REAL*8 CCCU(3,3,3,3),CCCL(3,3,3,3)
      REAL*8 CMU(6,6),BU(0:6,0:6,0:6)       
      REAL*8 CML(6,6),BL(0:6,0:6,0:6) 
      complex*16 S1P(0:6),S2P(0:6),S3PU(0:6),S3PL(0:6)   
      complex*16 E3U(0:6),E3L(0:6) 
      complex*16 AU(7),AAU(7),ACU(4)       
      complex*16 AL(7),AAL(7),ACL(4) 
      COMPLEX*16 S3U(6),UCU(6,3),S3CU(3),UCMU(3,3)       
      COMPLEX*16 S3L(6),UCL(6,3),S3CL(3),UCML(3,3)  
      COMPLEX*16 G(6,6),H(3),SMU(3,3),SML(3,3),hnw
      COMPLEX*16 XII,s1,s2
      INTEGER ipar(2)
      INTEGER IVNU(3),IVNL(3)
      LOGICAL polish
      common /cipar/ ipar
      common /cdep/ del,eps
C  ==============================
      polish=.true.
      xii=(0.0d0,1.0d0)
      PI=3.141592653589793D0
      twpi=2.0d0*pi      
C  =================================================
C     CALCULATION OF POWERS OF S1 AND S2 AND OF ARRAY A
      DO 570 K=1,6
      S1P(K)=S1P(K-1)*S1
c	  write(6,*) s1p(k)
      S2P(K)=S2P(K-1)*S2
570   CONTINUE
      DO 571 K=0,6
      E3U(K)=0.0D0
      E3L(K)=0.0D0
      DO 572 L=0,6,2
      IF(K.GT.L)GO TO 573
      DO 574 M=0,L-K
      N=L-M-K
      BMNK=BU(M,N,K)
      IF(BMNK.EQ.0.0D0)GO TO 574
      E3U(K)=E3U(K)+BU(M,N,K)*S1P(M)*S2P(N)            
574   CONTINUE
      DO 575 M=0,L-K
      N=L-M-K
      BMNK=BL(M,N,K)
      IF(BMNK.EQ.0.0D0)GO TO 575
      E3L(K)=E3L(K)+BL(M,N,K)*S1P(M)*S2P(N)            
575   CONTINUE
573   CONTINUE
572   CONTINUE
      AU(7-K)=E3U(K)/BU(0,0,6)
      AAU(K+1)=E3U(K)/BU(0,0,6)
      AL(7-K)=E3L(K)/BL(0,0,6)
      AAL(K+1)=E3L(K)/BL(0,0,6)
571   CONTINUE
C     FOR SZ PERPENDICULAR TO A MIRROR
C     PLANE, EQUATION OF DEGREE 3 IN SZ**2
      ACU(4)=AU(1)
      ACU(3)=AU(3)
      ACU(2)=AU(5)
      ACU(1)=AU(7)
      ACL(4)=AL(1)
      ACL(3)=AL(3)
      ACL(2)=AL(5)
      ACL(1)=AL(7)
c     Solving for Sz
c      call zroots(aa,6,s3,polish)
      call cubrt(acu,s3cu)
      S3U(1)=CDSQRT(S3CU(1))
      S3U(2)=-s3U(1)
      S3U(3)=CDSQRT(S3CU(2))
      S3U(4)=-s3U(3)
      S3U(5)=CDSQRT(S3CU(3))
      S3U(6)=-s3U(5)
      call cubrt(acl,s3cl)
      S3L(1)=CDSQRT(S3CL(1))
      S3L(2)=-s3L(1)
      S3L(3)=CDSQRT(S3CL(2))
      S3L(4)=-s3L(3)
      S3L(5)=CDSQRT(S3CL(3))
      S3L(6)=-s3L(5)
123   CONTINUE

C     ===============================================================
C     CALCULATION OF EIGENVECTORS UC(IBR,J) FOR 6 SOLUTIONS
      CALL SUB2(cu,rhou,ucu,s1,s2,s3u)         
      CALL SUB2(cl,rhol,ucl,s1,s2,s3l)   
C     ===============================================================
C     CALCULATION OF NUMBER NRW OF OUTGOING REAL WAVES AND NCW OF
C     OUTGOING COMPLEX WAVES. TOTAL NUMBER OF OUTGOING WAVES ivn(1-3)
      CALL SUB3(1,bu,ivnu,s3u,e3u,s1p,s2p,s1,s2)          
      CALL SUB3(-1,bl,ivnl,s3l,e3l,s1p,s2p,s1,s2)   
	         write(6,*) ivnl   
C     ===============================================================
      DO 773 IBR=1,3
      SMU(IBR,1)=S1
      SMU(IBR,2)=S2
      SMU(IBR,3)=S3U(ivnu(IBR))
      SML(IBR,1)=S1
      SML(IBR,2)=S2
      SML(IBR,3)=S3L(ivnl(IBR))
      do 7731 jbr=1,3
      ucmu(ibr,jbr)=ucu(ivnu(ibr),jbr)
      ucml(ibr,jbr)=ucl(ivnl(ibr),jbr)
7731  continue
773   CONTINUE
C     ============================================================
C     CALCULATION OF G(IX,IY),and solution of boundary equations to get
C     wave amplitudes h(i) and Green's function hnw
      CALL SUB4(cu,cl,rhou,rhol,ucmu,ucml,smu,sml,h)         
C     ============================================================
      hnw=(0.0d0,0.0d0)      
      do 44477 nw=1,3
      hnw=hnw+h(nw)
44477  continue
      return
      end


C     ************************************************************
C     CALCULATION OF G(IX,IY),DETERMINANTS,AMPLITUDES 
C     OF OUTGOING WAVES H(I)
      SUBROUTINE SUB4(cu,cl,rhou,rhol,ucmu,ucml,smu,sml,h)
C     ************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 CU(3,3,3,3),CL(3,3,3,3),gr(12,12),deltafr(12,3)
      INTEGER ipar(2)
      COMPLEX*16 G(6,6),H(3),SMU(3,3),SML(3,3),xii
      complex*16 ucmu(3,3),ucml(3,3),deltaf(6,3)
      common /cipar/ ipar
c
      igf=ipar(1)
      jgf=ipar(2)
      xii=(0.0d0,1.0d0)
      do 210 i=1,12
      do 210 j=1,3
      deltafr(i,j)=0.0d0
210   continue
      deltafr(1,1)=1.0d0        
      deltafr(2,2)=1.0d0        
      deltafr(3,3)=1.0d0  
      do 211 i=1,6
      do 211 j=1,3
      deltaf(i,j)=(0.0d0,0.0d0)
211    continue
      deltaf(1,1)=(1.0d0,0.0d0)
      deltaf(2,2)=(1.0d0,0.0d0)
      deltaf(3,3)=(1.0d0,0.0d0)
c     ============================================
      DO 774 IX=1,3
      DO 774 IY=1,3
      G(IX,IY)=0.0D0
      G(IX,IY+3)=0.0d0
      DO 775 IP=1,3
      DO 775 IQ=1,3
      C3XPQ=CU(3,IX,IP,IQ)
      IF(C3XPQ.EQ.0.0D0)GO TO 775
      G(IX,IY)=G(IX,IY)
     1 +CU(3,IX,IP,IQ)*UCmu(iy,IP)*SMU(IY,IQ)*rhou
775   CONTINUE
      DO 776 IP=1,3
      DO 776 IQ=1,3
      C3XPQ=CL(3,IX,IP,IQ)
      IF(C3XPQ.EQ.0.0D0)GO TO 776
      G(IX,IY+3)=G(IX,IY+3)
     1 -CL(3,IX,IP,IQ)*UCml(iy,IP)*SML(IY,IQ)*rhol
776   CONTINUE
774   CONTINUE
      DO 801 IP=1,3
      DO 801 JP=1,3
      G(IP+3,JP)=UCMU(JP,IP)
      G(IP+3,JP+3)=-UCML(JP,IP)
801   continue
c     =============================================
c     g is entered and its inverse is returned as g
c      call gaussj2(g,6,6,deltaf,3,3)      
      do 310 i=1,6      
      do 310 j=1,6                                    
      gr(i,j)=dimag(xii*g(i,j))
      gr(i,j+6)=-dimag(g(i,j))
      gr(i+6,j)=dimag(g(i,j))
      gr(i+6,j+6)=dimag(xii*g(i,j))
310   continue
      call gaussj(gr,12,12,deltafr,3,3)
c     =============================================      
      do 870 ii=1,3
      h(ii)=ucmu(ii,igf)*(deltafr(ii,jgf)+xii*deltafr(ii+6,jgf))
870   continue
      RETURN
      END





C     *************************************************************
C     CALCULATION OF GAM AND EIGENVECTORS UC(IBR,J) FOR 6 SOLUTIONS
      SUBROUTINE SUB2(c,rho,uc,s1,s2,s3)
C     *************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 C(3,3,3,3)
      COMPLEX*16 S3(6),GAM(3,3),S(3),UC(6,3),s1,s2
      S(1)=S1   
      S(2)=S2
      DO 670 IBR=1,6
      S(3)=S3(IBR)
      DO 671 K=1,3
      DO 671 L=1,3
      GAM(K,L)=0.0D0
      DO 672 M=1,3
      DO 672 N=1,3
      CKMLN=C(K,M,L,N)
      IF(CKMLN.EQ.0.0D0)GO TO 672
      GAM(K,L)=GAM(K,L)+C(K,M,L,N)*S(M)*S(N)
672   CONTINUE
671   CONTINUE
      UC(IBR,1)=GAM(1,2)*GAM(2,3)-GAM(1,3)*(GAM(2,2)-1.0D0)
      UC(IBR,2)=GAM(1,3)*GAM(2,1)-GAM(2,3)*(GAM(1,1)-1.0D0)
      UC(IBR,3)=(GAM(1,1)-1.0D0)*(GAM(2,2)-1.0D0)-GAM(1,2)*GAM(2,1)
670   CONTINUE
      RETURN
      END


C     ****************************************************************
C     CALCULATION OF NUMBER NRW OF OUTGOING REAL WAVES AND NCW OF
C     OUTGOING COMPLEX WAVES. TOTAL NUMBER OF OUTGOING WAVES=3
      SUBROUTINE SUB3(isp,b,ivn,s3,e3,s1p,s2p,s1,s2)
C     ****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      complex*16 S1P(0:6),S2P(0:6),S3P(0:6)
      complex*16 e1(0:6),e2(0:6),E3(0:6)
      real*8 B(0:6,0:6,0:6)
      COMPLEX*16 S3(6),xii
      INTEGER IVN(3),IR(3),JR(3)
      NRW=0
      NCW=0
      NVN=0
      NR=0
      xii=dcmplx(0.0d0,1.0d0)
      s3p(0)=dcmplx(1.0d0,0.0d0)
      DO 772 IBR=1,6
      if(dabs(dimag(s3(ibr))).gt.0.00000001d0)goto 770
c
      s3(ibr)=s3(ibr)-xii*dimag(s3(ibr))
      DO 575 K=1,6
      S3P(K)=S3P(K-1)*S3(IBR)
575   CONTINUE
      do 576 k=0,6
      e1(k)=0.0d0
      e2(k)=0.0d0
      do 577 l=0,6,2
      if(k.gt.l) go to 578
      do 579 m=0,l-k
      n=l-m-k
      bkmn=b(k,m,n)
      if(bkmn.eq.0.0d0)go to 5791
      e1(k)=e1(k)+bkmn*s2p(m)*s3p(n)
5791  bmkn=b(m,k,n)
      if(bmkn.eq.0.0d0)go to 579
      e2(k)=e2(k)+bmkn*s1p(m)*s3p(n)
579   continue
578   continue
577   continue
576   continue
c
      VGMIBR3=6.D0*E3(6)*S3P(5)+5.D0*E3(5)*S3P(4)+4.D0*E3(4)*S3P(3)
     1          +3.D0*E3(3)*S3P(2)+2.D0*E3(2)*S3P(1)+1.D0*E3(1)*S3P(0)
      vgmibr2=6.0d0*e2(6)*s2p(5)+5.0d0*e2(5)*s2p(4)+4.0d0*e2(4)*s2p(3)
     1       +3.0d0*e2(3)*s2p(2)+2.0d0*e2(2)*s2p(1)+1.0d0*e2(1)*s2p(0)
      vgmibr1=6.0d0*e1(6)*s1p(5)+5.0d0*e1(5)*s1p(4)+4.0d0*e1(4)*s1p(3)
     1       +3.0d0*e1(3)*s1p(2)+2.0d0*e1(2)*s1p(1)+1.0d0*e1(1)*s1p(0)
      sdvg=s1*vgmibr1+s2*vgmibr2+s3(ibr)*vgmibr3
      vgmibr3=vgmibr3/sdvg
c
      IF(isp.eq.1.and.dimag(xii*VGMIBR3).GT.0.0D0)NRW=NRW+1
      IF(isp.eq.1.and.dimag(xii*VGMIBR3).GT.0.0D0)IR(NRW)=IBR
      IF(isp.eq.-1.and.dimag(xii*VGMIBR3).LT.0.0D0)NRW=NRW+1
      IF(isp.eq.-1.and.dimag(xii*VGMIBR3).LT.0.0D0)IR(NRW)=IBR
c
      GO TO 771
770   CONTINUE
      IF(isp.eq.1.and.DIMAG(S3(IBR)).GT.0.0D0)NCW=NCW+1
      IF(isp.eq.1.and.DIMAG(S3(IBR)).GT.0.0D0)JR(NCW)=IBR
      IF(isp.eq.-1.and.DIMAG(S3(IBR)).LT.0.0D0)NCW=NCW+1
      IF(isp.eq.-1.and.DIMAG(S3(IBR)).LT.0.0D0)JR(NCW)=IBR
771   CONTINUE
772   CONTINUE
      if(ncw.eq.0)then
      ivn(1)=ir(1)
      ivn(2)=ir(2)
      ivn(3)=ir(3)
      end if
      if(ncw.eq.1)then
      ivn(1)=ir(1)
      ivn(2)=ir(2)
      ivn(3)=jr(1)
      end if
      if(ncw.eq.2)then
      ivn(1)=ir(1)
      ivn(2)=jr(1)
      ivn(3)=jr(2)
      end if
      if(ncw.eq.3)then
      ivn(1)=jr(1)
      ivn(2)=jr(2)
      ivn(3)=jr(3)
      end if
      RETURN
      END


      SUBROUTINE SINIT(atr,b,cm,ccc,c)
C     *******************************
C
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 C(3,3,3,3),DEL(3,3),EPS(3,3,3)
      REAL*8 atr(3,3),ccc(3,3,3,3)
      REAL*8 a4(3,3,3,3),a6(3,3,3,3,3,3)
      REAL*8 B(0:6,0:6,0:6),CM(6,6)
      INTEGER II(9)
      common /cdep/ del,eps
C     ********************************************************
c     Tensor elastic constants in crystal frame
      do 63 i=1,3
      do 63 j=1,3
      do 63 k=1,3
      do 63 l=1,3
      m=9-i-j
      if(i.eq.j)m=i
      n=9-k-l
      if(k.eq.l)n=k
      ccc(i,j,k,l)=cm(m,n)
 63   continue
c ==========================================
c     Tensor elastic constants in lab frame
      do 663 i=1,3
      do 663 j=1,3
      do 663 k=1,3
      do 663 l=1,3
      c(i,j,k,l)=0.0d0
      do 664 ia=1,3
      do 664 ja=1,3
      do 664 ka=1,3
      do 664 la=1,3
      zy=atr(i,ia)*atr(j,ja)*atr(k,ka)*atr(l,la)*ccc(ia,ja,ka,la)
      c(i,j,k,l)=c(i,j,k,l)+zy
 664  continue
 663  continue
c  ==========================================================
c     Kronecker delta and alternating tensors
      do 3 i=1,3
      do 3 j=1,3
      del(i,j)=0.0d0
      do 3 k=1,3
      eps(i,j,k)=0.0d0
 3    continue
      del(1,1)=1.0d0
      del(2,2)=1.0d0
      del(3,3)=1.0d0
      eps(1,2,3)=1.0d0
      eps(2,3,1)=1.0d0
      eps(3,1,2)=1.0d0
      eps(3,2,1)=-1.0d0
      eps(2,1,3)=-1.0d0
      eps(1,3,2)=-1.0d0

C     ==========================================================
C     Procedure for calculating B(i,j,k), the matrix of coefficients
c     of the equation of the slowness surface
      DO 31 I=1,3
      DO 31 J=1,3
      DO 31 K=1,3
      DO 31 L=1,3
      A4(I,J,K,L)=0.0D0
      DO 34 IP=1,3
      DO 34 IQ=1,3
      DO 34 IRR=1,3
      DO 34 IS=1,3
      DO 34 IT=1,3
      A4(I,J,K,L)=A4(I,J,K,L)+EPS(IP,IQ,IT)*EPS(IRR,IS,IT)*
     1C(IP,I,IRR,J)*C(IQ,K,IS,L)/2.0D0
34    CONTINUE
      DO 32 M=1,3
      DO 32 N=1,3
      A6(I,J,K,L,M,N)=0.0D0
      DO 33 IP=1,3
      DO 33 IQ=1,3
      DO 33 IRR=1,3
      A6(I,J,K,L,M,N)=A6(I,J,K,L,M,N)+EPS(IP,IQ,IRR)*
     1C(1,I,IP,J)*C(2,K,IQ,L)*C(3,M,IRR,N)
33    CONTINUE
32    CONTINUE
31    CONTINUE
C
      DO 30 I=0,6
      DO 30 J=0,6
      DO 30 K=0,6
      B(I,J,K)=0.0D0
30    CONTINUE
C
      DO 40 I=0,6
      DO 40 J=0,6-I
      K=6-I-J
      DO 41 IL=1,3
      DO 41 IM=1,3
      DO 41 IP=1,3
      DO 41 IQ=1,3
      DO 41 IR=1,3
      DO 41 IS=1,3
      II(1)=0
      II(2)=0
      II(3)=0
      II(IL)=II(IL)+1
      II(IM)=II(IM)+1
      II(IP)=II(IP)+1
      II(IQ)=II(IQ)+1
      II(IR)=II(IR)+1
      II(IS)=II(IS)+1
      IF(I.EQ.II(1).AND.J.EQ.II(2).AND.K.EQ.II(3))
     1B(I,J,K)=B(I,J,K)+A6(IL,IM,IP,IQ,IR,IS)
41    CONTINUE
40    CONTINUE
      DO 43 I=0,4
      DO 43 J=0,4-I
      K=4-I-J
      DO 44 IL=1,3
      DO 44 IM=1,3
      DO 44 IR=1,3
      DO 44 IS=1,3
      II(1)=0
      II(2)=0
      II(3)=0
      II(IL)=II(IL)+1
      II(IM)=II(IM)+1
      II(IR)=II(IR)+1
      II(IS)=II(IS)+1
      IF(I.EQ.II(1).AND.J.EQ.II(2).AND.K.EQ.II(3))
     1B(I,J,K)=B(I,J,K)-A4(IL,IM,IR,IS)
44    CONTINUE
43    CONTINUE
      B(2,0,0)=C(1,1,1,1)+C(2,1,2,1)+C(3,1,3,1)
      B(0,2,0)=C(1,2,1,2)+C(2,2,2,2)+C(3,2,3,2)
      B(0,0,2)=C(1,3,1,3)+C(2,3,2,3)+C(3,3,3,3)
      B(1,1,0)=C(1,1,1,2)+C(2,1,2,2)+C(3,1,3,2)
     1        +C(1,2,1,1)+C(2,2,2,1)+C(3,2,3,1)
      B(1,0,1)=C(1,1,1,3)+C(2,1,2,3)+C(3,1,3,3)
     1        +C(1,3,1,1)+C(2,3,2,1)+C(3,3,3,1)
      B(0,1,1)=C(1,2,1,3)+C(2,2,2,3)+C(3,2,3,3)
     1        +C(1,3,1,2)+C(2,3,2,2)+C(3,3,3,2)
C
      B(0,0,0)=-1.0D0
C
c     =================================
      RETURN
      END


      subroutine cubrt(a,cz)
c
c     Roots of a cubic equation a(i)z**(i-1)
c     Z**3+a2*Z**2+a1*Z+A0 as in Abramowitz and Stegun
c
      real*8 third,sn60,twn7
      complex*16 a(4),cz(3)
      complex*16 a0,a1,a2,g,h,a23,sud,q,omeg,omeg2,g2,h4,hq
      third=1.0d0/3.0d0
      twn7=1.0d0/27.0d0
      sn60=0.5d0*dsqrt(3.0d0)
c      write(6,*)third,twn7,sn60
      a2=a(3)/a(4)
      a1=a(2)/a(4)
      a0=a(1)/a(4)
      a23=a2*third
      h=(-a2*a2*third+a1)*third
      g=2.0d0*a2*a2*a2*twn7-a1*a2*third+a0
      g2=g*g
      h4=4.0d0*h*h*h
      if(cdabs(h4).gt.0.000000001d0*cdabs(g2))then
      sud=(-g+cdsqrt(g2+h4))/2.0d0
      else
      sud=h4/(4.0d0*g)
      end if
      q=sud**third
      omeg=dcmplx(-0.5d0,sn60)
      omeg2=dcmplx(-0.5d0,-sn60)
      hq=h/q
      cz(1)=q-hq-a23
      cz(2)=omeg*q-omeg2*hq-a23
      cz(3)=omeg2*q-omeg*hq-a23
c      write(6,*)omeg,omeg*omeg*omeg
c      write(6,*)omeg2,omeg2*omeg2*omeg2
      return
      end

      SUBROUTINE GAUSSJ(A,N,NP,B,M,MP)
      PARAMETER (NMAX=12)
      IMPLICIT REAL*8 (A-H,O-Z)
      real*8 A(NP,NP),B(NP,MP) 
      INTEGER IPIV(NMAX),INDXR(NMAX),INDXC(NMAX)
      DO 11 J=1,N
        IPIV(J)=0
11    CONTINUE
      DO 22 I=1,N
        BIG=0.0d0
        DO 13 J=1,N
          IF(IPIV(J).NE.1)THEN
            DO 12 K=1,N
              IF (IPIV(K).EQ.0) THEN
                IF (DABS(A(J,K)).GE.BIG)THEN
                  BIG=DABS(A(J,K))
                  IROW=J
                  ICOL=K
                ENDIF
              ELSE IF (IPIV(K).GT.1) THEN
                PAUSE 'Singular matrix'
              ENDIF
12          CONTINUE
          ENDIF
13      CONTINUE
        IPIV(ICOL)=IPIV(ICOL)+1
        IF (IROW.NE.ICOL) THEN
          DO 14 L=1,N
            DUM=A(IROW,L)
            A(IROW,L)=A(ICOL,L)
            A(ICOL,L)=DUM
14        CONTINUE
          DO 15 L=1,M
            DUM=B(IROW,L)
            B(IROW,L)=B(ICOL,L)
            B(ICOL,L)=DUM
15        CONTINUE
        ENDIF
        INDXR(I)=IROW
        INDXC(I)=ICOL
        IF (A(ICOL,ICOL).EQ.0.0D0) PAUSE 'Singular matrix.'
        PIVINV=1.0d0/A(ICOL,ICOL)
        A(ICOL,ICOL)=1.0D0
        DO 16 L=1,N
          A(ICOL,L)=A(ICOL,L)*PIVINV
16      CONTINUE
        DO 17 L=1,M
          B(ICOL,L)=B(ICOL,L)*PIVINV
17      CONTINUE
        DO 21 LL=1,N
          IF(LL.NE.ICOL)THEN
            DUM=A(LL,ICOL)
            A(LL,ICOL)=0.0D0
            DO 18 L=1,N
              A(LL,L)=A(LL,L)-A(ICOL,L)*DUM
18          CONTINUE
            DO 19 L=1,M
              B(LL,L)=B(LL,L)-B(ICOL,L)*DUM
19          CONTINUE
          ENDIF
21      CONTINUE
22    CONTINUE
      DO 24 L=N,1,-1
        IF(INDXR(L).NE.INDXC(L))THEN
          DO 23 K=1,N
            DUM=A(K,INDXR(L))
            A(K,INDXR(L))=A(K,INDXC(L))
            A(K,INDXC(L))=DUM
23        CONTINUE
        ENDIF
24    CONTINUE
      RETURN
      END


      SUBROUTINE GAUSSJ2(A,N,NP,B,M,MP)
      PARAMETER (NMAX=6)
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 DUM,PIVINV
      COMPLEX*16 A(NP,NP),B(NP,MP) 
      INTEGER IPIV(NMAX),INDXR(NMAX),INDXC(NMAX)
      DO 11 J=1,N
        IPIV(J)=0
11    CONTINUE
      DO 22 I=1,N
        BIG=0.0d0
        DO 13 J=1,N
          IF(IPIV(J).NE.1)THEN
            DO 12 K=1,N
              IF (IPIV(K).EQ.0) THEN
                IF (CDABS(A(J,K)).GE.BIG)THEN
                  BIG=CDABS(A(J,K))
                  IROW=J
                  ICOL=K
                ENDIF
              ELSE IF (IPIV(K).GT.1) THEN
                PAUSE 'Singular matrix'
              ENDIF
12          CONTINUE
          ENDIF
13      CONTINUE
        IPIV(ICOL)=IPIV(ICOL)+1
        IF (IROW.NE.ICOL) THEN
          DO 14 L=1,N
            DUM=A(IROW,L)
            A(IROW,L)=A(ICOL,L)
            A(ICOL,L)=DUM
14        CONTINUE
          DO 15 L=1,M
            DUM=B(IROW,L)
            B(IROW,L)=B(ICOL,L)
            B(ICOL,L)=DUM
15        CONTINUE
        ENDIF
        INDXR(I)=IROW
        INDXC(I)=ICOL
        IF (A(ICOL,ICOL).EQ.(0.0D0,0.0D0)) PAUSE 'Singular matrix.'
        PIVINV=1.0d0/A(ICOL,ICOL)
        A(ICOL,ICOL)=1.0D0
        DO 16 L=1,N
          A(ICOL,L)=A(ICOL,L)*PIVINV
16      CONTINUE
        DO 17 L=1,M
          B(ICOL,L)=B(ICOL,L)*PIVINV
17      CONTINUE
        DO 21 LL=1,N
          IF(LL.NE.ICOL)THEN
            DUM=A(LL,ICOL)
            A(LL,ICOL)=0.0D0
            DO 18 L=1,N
              A(LL,L)=A(LL,L)-A(ICOL,L)*DUM
18          CONTINUE
            DO 19 L=1,M
              B(LL,L)=B(LL,L)-B(ICOL,L)*DUM
19          CONTINUE
          ENDIF
21      CONTINUE
22    CONTINUE
      DO 24 L=N,1,-1
        IF(INDXR(L).NE.INDXC(L))THEN
          DO 23 K=1,N
            DUM=A(K,INDXR(L))
            A(K,INDXR(L))=A(K,INDXC(L))
            A(K,INDXC(L))=DUM
23        CONTINUE
        ENDIF
24    CONTINUE
      RETURN
      END
	  
	  	  
C   ***************************************************************************************
      subroutine hvalhs(s1,s2,vel,intns,bu,bl,cu,cl,rhou,rhol,
     1 damping,mflg)   
C   *****************************************************************************
c*************************      
c     Calculation of intensities  (half spaces in contact)
c*************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 CU(3,3,3,3),CL(3,3,3,3),DEL(3,3),EPS(3,3,3)
      REAL*8 BU(0:6,0:6,0:6)       
      REAL*8 BL(0:6,0:6,0:6) 
      complex*16 S1P(0:6),S2P(0:6),S3PU(0:6),S3PL(0:6)   
      complex*16 E3U(0:6),E3L(0:6) 
      complex*16 AAU(7),ACU(4)       
      complex*16 AAL(7),ACL(4) 
	  complex*16 AU(7)
      complex*16 AL(7)
      COMPLEX*16 S3U(6),UCU(6,3),S3CU(3),UCMU(3,3)       
      COMPLEX*16 S3L(6),UCL(6,3),S3CL(3),UCML(3,3)  
      COMPLEX*16 G(6,6),H(3),SMU(3,3),SML(3,3),hnw
      COMPLEX*16 XII,s1,s2
      INTEGER ipar(2),mflg
	  real*8 intns,damping,vel,rhou,rhol
      INTEGER IVNU(3),IVNL(3)
      LOGICAL polish
      common /cipar/ ipar
      common /cdep/ del,eps
C  ==============================

      S1P(0)=(1.0D0,0.0d0)
      S2P(0)=(1.0D0,0.0d0)
      polish=.true.
      xii=(0.0d0,1.0d0)
      PI=3.141592653589793D0
      twpi=2.0d0*pi      
C  =================================================
C     CALCULATION OF POWERS OF S1 AND S2 AND OF ARRAY A
      DO 570 K=1,6
      S1P(K)=S1P(K-1)*S1
	 write(6,*) s1p(k)
      S2P(K)=S2P(K-1)*S2
570   CONTINUE
      DO 571 K=0,6
      E3U(K)=0.0D0
      E3L(K)=0.0D0
      DO 572 L=0,6,2
      IF(K.GT.L)GO TO 573
      DO 574 M=0,L-K
      N=L-M-K
      BMNK=BU(M,N,K)
      IF(BMNK.EQ.0.0D0)GO TO 574
      E3U(K)=E3U(K)+BU(M,N,K)*S1P(M)*S2P(N)            
574   CONTINUE
      DO 575 M=0,L-K
      N=L-M-K
      BMNK=BL(M,N,K)
      IF(BMNK.EQ.0.0D0)GO TO 575
      E3L(K)=E3L(K)+BL(M,N,K)*S1P(M)*S2P(N)            
575   CONTINUE
573   CONTINUE
572   CONTINUE
      AU(7-K)=E3U(K)/BU(0,0,6)
      AAU(K+1)=E3U(K)/BU(0,0,6)
      AL(7-K)=E3L(K)/BL(0,0,6)
      AAL(K+1)=E3L(K)/BL(0,0,6)
571   CONTINUE

       if (mflg .EQ. 1) then
C     FOR SZ PERPENDICULAR TO A MIRROR
C     PLANE, EQUATION OF DEGREE 3 IN SZ**2
        ACU(4)=AU(1)
        ACU(3)=AU(3)
        ACU(2)=AU(5)
        ACU(1)=AU(7)
        ACL(4)=AL(1)
        ACL(3)=AL(3)
        ACL(2)=AL(5)
        ACL(1)=AL(7)
        call cubrt(acu,s3cu)
        S3U(1)=CDSQRT(S3CU(1))
        S3U(2)=-s3U(1)
        S3U(3)=CDSQRT(S3CU(2))
        S3U(4)=-s3U(3)
        S3U(5)=CDSQRT(S3CU(3))
        S3U(6)=-s3U(5)
        call cubrt(acl,s3cl)
        S3L(1)=CDSQRT(S3CL(1))
        S3L(2)=-s3L(1)
        S3L(3)=CDSQRT(S3CL(2))
        S3L(4)=-s3L(3)
        S3L(5)=CDSQRT(S3CL(3))
        S3L(6)=-s3L(5)
      else
c    non-mirror plane
       call zroots(aau,6,s3u,polish)
       call zroots(aal,6,s3l,polish)
	  endif
123   CONTINUE

C     ===============================================================
C     CALCULATION OF EIGENVECTORS UC(IBR,J) FOR 6 SOLUTIONS
            write(6,*) 'here'
      CALL SUB2(cu,ucu,s1,s2,s3u)      

      CALL SUB2(cl,ucl,s1,s2,s3l)   

C     ===============================================================
C     CALCULATION OF NUMBER NRW OF OUTGOING REAL WAVES AND NCW OF
C     OUTGOING COMPLEX WAVES. TOTAL NUMBER OF OUTGOING WAVES ivn(1-3)
            write(6,*) 'here'
      CALL SUB3(1,bu,ivnu,s3u,e3u,s1p,s2p,s1,s2)      
            write(6,*) ivnu 
      CALL SUB3(-1,bl,ivnl,s3l,e3l,s1p,s2p,s1,s2)    
	              write(6,*) ivnl

C     ===============================================================
      DO 773 IBR=1,3
      SMU(IBR,1)=S1
      SMU(IBR,2)=S2
      SMU(IBR,3)=S3U(ivnu(IBR))
      SML(IBR,1)=S1
      SML(IBR,2)=S2
      SML(IBR,3)=S3L(ivnl(IBR))
      do 7731 jbr=1,3
      ucmu(ibr,jbr)=ucu(ivnu(ibr),jbr)
      ucml(ibr,jbr)=ucl(ivnl(ibr),jbr)
7731  continue
773   CONTINUE
C     ============================================================
C     CALCULATION OF G(IX,IY),and solution of boundary equations to get
C     wave amplitudes h(i) and Green's function hnw
            write(6,*) 'here'
      CALL SUB4hs(cu,cl,rhou,rhol,ucmu,ucml,smu,sml,h,iflg)         
C     ============================================================
            write(6,*) 'here2'
      hnw=(0.0d0,0.0d0)      
      do 44477 nw=1,3
      hnw=hnw+h(nw)
	              write(6,*) ipar
44477  continue
c	   intns=1.0d4*dimag(xii/(vel*vel*(1.0d0/hnw+damping)))
	   if((ipar(1) .EQ. 3) .AND. (ipar(2) .EQ. 3)) then
	    intns=1.0d4*dimag(xii/(vel*vel*(1.0d0/hnw+damping)))
	   else if ((((ipar(1) .EQ. 1) .AND. (ipar(2) .EQ. 3)) .OR.
     1  ((ipar(1) .EQ. 3) .AND. (ipar(2) .EQ. 1)))) then
        intns=1.0d4*dimag(xii/(vel*vel*cdabs(1.0d0/hnw+damping)))
		else if ((((ipar(1) .EQ. 1) .AND. (ipar(2) .EQ. 2)) .OR.
     1  ((ipar(1) .EQ. 2) .AND. (ipar(2) .EQ. 1)))) then
        intns=1.0d4*dimag(xii/(vel*vel*cdabs(1.0d0/hnw+damping)))
	   endif
		              write(6,*) intns
	   if (iflg .EQ. 1) intns=0.d0
      return
      end


C     ***************************************************************************************
      SUBROUTINE SUB4hs(cu,cl,rhou,rhol,ucmu,ucml,smu,sml,h,iflg)
C     CALCULATION OF G(IX,IY),DETERMINANTS,AMPLITUDES 
C     OF OUTGOING WAVES H(I)   (Half spaces in contact)
C     ************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 CU(3,3,3,3),CL(3,3,3,3),gr(12,12),deltafr(12,3)
      INTEGER ipar(2),iflg
      COMPLEX*16 G(6,6),H(3),SMU(3,3),SML(3,3),xii
      complex*16 ucmu(3,3),ucml(3,3),deltaf(6,3)
      common /cipar/ ipar
c
      igf=ipar(1)
      jgf=ipar(2)
      xii=(0.0d0,1.0d0)
      do 210 i=1,12
      do 210 j=1,3
      deltafr(i,j)=0.0d0
210   continue
      deltafr(1,1)=1.0d0        
      deltafr(2,2)=1.0d0        
      deltafr(3,3)=1.0d0  
      do 211 i=1,6
      do 211 j=1,3
      deltaf(i,j)=(0.0d0,0.0d0)
211    continue
      deltaf(1,1)=(1.0d0,0.0d0)
      deltaf(2,2)=(1.0d0,0.0d0)
      deltaf(3,3)=(1.0d0,0.0d0)
c     ============================================
      DO 774 IX=1,3
      DO 774 IY=1,3
      G(IX,IY)=0.0D0
      G(IX,IY+3)=0.0d0
      DO 775 IP=1,3
      DO 775 IQ=1,3
      C3XPQ=CU(3,IX,IP,IQ)
      IF(C3XPQ.EQ.0.0D0)GO TO 775
      G(IX,IY)=G(IX,IY)
     1 +CU(3,IX,IP,IQ)*UCmu(iy,IP)*SMU(IY,IQ)*rhou
775   CONTINUE
      DO 776 IP=1,3
      DO 776 IQ=1,3
      C3XPQ=CL(3,IX,IP,IQ)
      IF(C3XPQ.EQ.0.0D0)GO TO 776
      G(IX,IY+3)=G(IX,IY+3)
     1 -CL(3,IX,IP,IQ)*UCml(iy,IP)*SML(IY,IQ)*rhol
776   CONTINUE
774   CONTINUE
      DO 801 IP=1,3
      DO 801 JP=1,3
      G(IP+3,JP)=UCMU(JP,IP)
      G(IP+3,JP+3)=-UCML(JP,IP)
801   continue
c     =============================================
c     g is entered and its inverse is returned as g
c      call gaussj2(g,6,6,deltaf,3,3,iflg)      
      do 310 i=1,6      
      do 310 j=1,6                                    
      gr(i,j)=dimag(xii*g(i,j))
      gr(i,j+6)=-dimag(g(i,j))
      gr(i+6,j)=dimag(g(i,j))
      gr(i+6,j+6)=dimag(xii*g(i,j))
310   continue
      call gaussj(gr,12,12,deltafr,3,3,iflg)
c     =============================================      
      do 870 ii=1,3
      h(ii)=ucmu(ii,igf)*(deltafr(ii,jgf)+xii*deltafr(ii+6,jgf))
870   continue
      RETURN
      END



C     **********************************************************************************
      SUBROUTINE ZROOTS(A,M,ROOTS,POLISH)
C     ****************************************************************
      PARAMETER (EPS=1.D-6,MAXM=101)
      COMPLEX*16 A(*),ROOTS(M),AD(MAXM),X,B,C,XI
      LOGICAL POLISH
      XI=DCMPLX(0.0d0,1.0d0)
      DO 11 J=1,M+1
        AD(J)=A(J)
11    CONTINUE
      DO 13 J=M,1,-1
        X=DCMPLX(0.,0.)
        CALL LAGUER(AD,J,X,EPS,.FALSE.)
        IF(DABS(DIMAG(X)).LE.2.*EPS**2*DABS(DIMAG(XI*X))) 
     1        X=DCMPLX(DIMAG(XI*X),0.)
        ROOTS(J)=X
        B=AD(J+1)
        DO 12 JJ=J,1,-1
          C=AD(JJ)
          AD(JJ)=B
          B=X*B+C
12      CONTINUE
13    CONTINUE
      IF (POLISH) THEN
        DO 14 J=1,M
          CALL LAGUER(A,M,ROOTS(J),EPS,.TRUE.)
14      CONTINUE
      ENDIF
      DO 16 J=2,M
        X=ROOTS(J)
        DO 15 I=J-1,1,-1
          IF(DIMAG(XI*ROOTS(I)).LE.DIMAG(XI*X))GO TO 10
          ROOTS(I+1)=ROOTS(I)
15      CONTINUE
        I=0
10      ROOTS(I+1)=X
16    CONTINUE
      RETURN
      END

C    **************************************************************************************
      SUBROUTINE LAGUER(A,M,X,EPS,POLISH)
C    ******************************************************************
      COMPLEX*16 A(*),X,DX,X1,B,D,F,G,H,SQ,GP,GM,G2,ZERO,XI
      LOGICAL POLISH
      PARAMETER (ZERO=(0.0d0,0.0d0),EPSS=6.D-12,MAXIT=100)
      DXOLD=CDABS(X)
      DO 12 ITER=1,MAXIT
        B=A(M+1)
        ERR=CDABS(B)
        D=ZERO
        F=ZERO
        ABX=CDABS(X)
        DO 11 J=M,1,-1
          F=X*F+D
          D=X*D+B
          B=X*B+A(J)
          ERR=CDABS(B)+ABX*ERR
11      CONTINUE
        ERR=EPSS*ERR
        IF(CDABS(B).LE.ERR) THEN
          DX=ZERO
          RETURN
        ELSE
          G=D/B
          G2=G*G
          H=G2-2.*F/B
          SQ=CDSQRT((M-1)*(M*H-G2))
          GP=G+SQ
          GM=G-SQ
          IF(CDABS(GP).LT.CDABS(GM)) GP=GM
          DX=M/GP
        ENDIF
        X1=X-DX
        IF(X.EQ.X1)RETURN
        X=X1 
        CDX=CDABS(DX)
        IF(ITER.GT.6.AND.CDX.GE.DXOLD)RETURN
        DXOLD=CDX
        IF(.NOT.POLISH)THEN
          IF(CDABS(DX).LE.EPS*CDABS(X))RETURN
        ENDIF
12    CONTINUE
      write(6,*) 'too many iterations'
      RETURN
      END

